<?php

/**
 * Implement hook_drush_command().
 */
function modulate_drush_command() {
  $items = array();

  $items['modulate'] = array(
    'callback' => 'modulate_drush_generate',
    'description' => 'Generates the module.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_DATABASE,
  );
  $items['modulate-cleanup'] = array(
    'callback' => 'modulate_drush_cleanup',
    'description' => 'Removes the generated items from the database',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_DATABASE,
  );
  return $items;
}

function _modulate_drush_create_function($function_name_parts, $php) {
  $function_name = implode('_', $function_name_parts);
  $function = "function $function_name() {";
  $function .= preg_replace('/^\s*<\?(php)?(.*)\?>$/s', '\2', $php);
  // This is sometimes necessary and it's just easier to add always.
  $function .= ';';
  $function .= "\n}";
  return $function;
}

function modulate_drush_generate() {
  $result = db_query('SELECT theme, module, delta, pages FROM {blocks} WHERE visibility = 2 AND status = 1');
  while ($block = db_fetch_object($result)) {
    $function_name = array('_modulate_block_visiblity', $block->theme, $block->module, str_replace('-', '_', $block->delta));
    $functions[] = _modulate_drush_create_function($function_name, $block->pages);
    $block_conditions[] = array(
      'theme' => $block->theme,
      'module' => $block->module,
      'delta' => $block->delta,
      'function_name' => $function_name,
    );
  }
  $block_conditions = var_export($block_conditions, TRUE);
  $php_format = db_result(db_query("SELECT format FROM {filter_formats} WHERE name = 'PHP code'"));
  $modulate_views_data = array();
  if (module_exists('views')) {
    foreach (views_get_all_views() as $view) {
      $view->init_display();
      foreach ($view->display as $display_id => $display) {
        foreach (array('header', 'footer', 'empty', 'arguments') as $option) {
          if (!$display->handler->is_defaulted($option)) {
            foreach ($display->handler->get_option($option) as $id => $data) {
              $function_name_parts = array('_modulate_views', &$key, $view->name, $display_id, $option, $id);
              $modulate_data = array('view' => $view->name, 'display_id' => $display_id, 'option' => $option, 'id' => $id);
              if ($option == 'arguments') {
                foreach (array('default_argument' => '($view, $argument)', 'validate' => '($view, $handler, $argument)') as $type => $header) {
                  $key = $type . '_type';
                  if (isset($data[$key]) && $data[$key] == 'php') {
                    $modulate_views_data[] = $modulate_data + array('key' => $key);
                    $code = isset($data[$type . '_options']['code']) ? $data[$type . '_options']['code'] : $data[$type . '_php'];
                    $functions[] = 'function '. implode('_', $function_name_parts) . $header .'{' . $code . '}';
                  }
                }
              }
              elseif ($php_format && isset($data['field']) && $data['field'] == 'area' && $data['table'] == 'views' && $data['format'] == $php_format) {
                $key = 'area';
                $modulate_views_data[] = $modulate_data + array('key' => $key);
                $functions[] = _modulate_drush_create_function($function_name_parts, $data['content']);
              }
            }
          }
        }
      }
    }
  }
  $functions = implode("\n\n", $functions);
  $modulate_views_data = var_export($modulate_views_data, TRUE);
  $file = <<<MODULE
<?php

include_once dirname(__FILE__) . '/modulate.inc';

function _modulate_block_conditions() {
  return $block_conditions;
}

function _modulate_views_data() {
  return $modulate_views_data;
}

$functions
MODULE;
  file_put_contents(dirname(__FILE__) . '/modulate.inc', $file);
}

function modulate_drush_cleanup() {
  db_query("UPDATE {blocks} SET visibility = 0, pages = '' WHERE visibility = 2 AND status = 1");
}
